==> org-lsst-ccs-subsystem-ocsbridge-1.4.0-sources.jar: org/lsst/ccs/subsystem/ocsbridge/util/AggregateStatus.java <==
package org.lsst.ccs.subsystem.ocsbridge.util;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import org.lsst.ccs.subsystem.ocsbridge.util.State.StateChangeListener;

/**
 * A collection of different state objects.
 *
 * @author tonyj
 */
public class AggregateStatus {

    private final Map<Class<? extends Enum>, State> states = new LinkedHashMap<>();
    private final List<State.StateChangeListener<? extends Enum>> listeners = new CopyOnWriteArrayList<>();
    private final List<FutureStatus> waiters = new CopyOnWriteArrayList<>();
    private final StateChangeListener internalStateChangeListener = new InternalStateChangeListener();

    public void add(State<?> state) {
        State oldState = states.get(state.getEnumClass());
        if (oldState == null) {
            states.put(state.getEnumClass(), state);
            state.addStateChangeListener(internalStateChangeListener);
            notifyStateChanged(state.getState(), null);
        } else {
            oldState.setState(state.getState());
        }
    }

    State ge(Enum type) {
        return states.get(type.getDeclaringClass());
    }

    /**
     * Test if all given states are present in the aggregate status.
     *
     * @param statesToTest The states to look for
     * @return <code>true</code> if all states are present
     */
    public boolean hasState(Enum... statesToTest) {
        for (Enum e : statesToTest) {
            Class<? extends Enum> enumClass = e.getClass();
            State state = states.get(enumClass);
            if (state == null || !state.isInState(e)) {
                return false;
            }
        }
        return true;
    }

    public Collection<State> getStates() {
        return states.values();
    }

    <T extends Enum> void notifyStateChanged(final T currentState, final T oldState) {
        for (StateChangeListener l : listeners) {
            l.stateChanged(currentState, oldState);
        }
        for (FutureStatus waiter : waiters) {
            if (hasState(waiter.state)) {
                waiter.done();
            }
        }
    }

    void addStateChangeListener(State.StateChangeListener<? extends Enum> listener) {
        listeners.add(listener);
    }

    void removeStateChangeListener(State.StateChangeListener<? extends Enum> listener) {
        listeners.remove(listener);
    }

    Future<Void> waitForStatus(Enum state) {
        FutureStatus waiter = new FutureStatus(state);
        if (hasState(state)) {
            waiter.done();
        } else {
            waiters.add(waiter);
        }
        return waiter;
    }

    @Override
    public String toString() {
        return "AggregateStatus{" + "states=" + states.values() + '}';
    }

    private class InternalStateChangeListener implements StateChangeListener {

        @Override
        public void stateChanged(Enum state, Enum oldState) {
            notifyStateChanged(state,oldState);
        }


    }

    /**
     * An implementation of a future which waits for a particular state.
     */
    private class FutureStatus implements Future<Void> {

        private boolean done = false;
        private boolean cancelled = false;
        private final Enum state;

        FutureStatus(Enum state) {
            this.state = state;
        }

        @Override
        public synchronized boolean cancel(boolean mayInterruptIfRunning) {
            cancelled = true;
            waiters.remove(this);
            notifyAll();
            return true;
        }

        @Override
        public boolean isCancelled() {
            return cancelled;
        }

        @Override
        public boolean isDone() {
            return done;
        }

        @Override
        public synchronized Void get() throws InterruptedException, ExecutionException {
            while (!done) {
                wait();
            }
            return null;
        }

        @Override
        public synchronized Void get(long timeout, TimeUnit unit) throws InterruptedException, ExecutionException, TimeoutException {
            wait(unit.toMillis(timeout));
            if (!done) {
                throw new TimeoutException("Timeout waiting for state: " + state);
            }
            return null;
        }

        synchronized void done() {
            done = true;
            notifyAll();
        }
    }

}

==> org-lsst-ccs-subsystem-ocsbridge-2.6.5-sources.jar: org/lsst/ccs/subsystem/ocsbridge/util/AggregateStatus.java <==
package org.lsst.ccs.subsystem.ocsbridge.util;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import org.lsst.ccs.subsystem.ocsbridge.util.State.StateChangeListener;
import org.lsst.ccs.utilities.taitime.CCSTimeStamp;

/**
 * A collection of different state objects.
 *
 * @author tonyj
 */
public class AggregateStatus {

    private final Map<Class<? extends Enum>, State> states = new LinkedHashMap<>();
    private final List<State.StateChangeListener<? extends Enum>> listeners = new CopyOnWriteArrayList<>();
    private final List<FutureStatus> waiters = new CopyOnWriteArrayList<>();
    private final StateChangeListener internalStateChangeListener = new InternalStateChangeListener();

    public void add(CCSTimeStamp when, State<?> state) {
        State oldState = states.get(state.getEnumClass());
        if (oldState == null) {
            states.put(state.getEnumClass(), state);
            state.addStateChangeListener(internalStateChangeListener);
            notifyStateChanged(when, state.getState(), null);
        } else {
            oldState.setState(when, state.getState());
        }
    }

    State get(Enum type) {
        return states.get(type.getDeclaringClass());
    }

    /**
     * Test if all given states are present in the aggregate status.
     *
     * @param statesToTest The states to look for
     * @return <code>true</code> if all states are present
     */
    public boolean hasState(Enum... statesToTest) {
        for (Enum e : statesToTest) {
            Class<? extends Enum> enumClass = e.getClass();
            State state = states.get(enumClass);
            if (state == null || !state.isInState(e)) {
                return false;
            }
        }
        return true;
    }

    public Collection<State> getStates() {
        return states.values();
    }

    <T extends Enum> void notifyStateChanged(CCSTimeStamp when, final T currentState, final T oldState) {
        for (StateChangeListener l : listeners) {
            l.stateChanged(when, currentState, oldState);
        }
        for (FutureStatus waiter : waiters) {
            if (hasState(waiter.state)) {
                waiter.done();
            }
        }
    }

    void addStateChangeListener(State.StateChangeListener<? extends Enum> listener) {
        listeners.add(listener);
    }

    void removeStateChangeListener(State.StateChangeListener<? extends Enum> listener) {
        listeners.remove(listener);
    }

    Future<Void> waitForStatus(Enum state) {
        FutureStatus waiter = new FutureStatus(state);
        if (hasState(state)) {
            waiter.done();
        } else {
            waiters.add(waiter);
        }
        return waiter;
    }

    @Override
    public String toString() {
        return "AggregateStatus{" + "states=" + states.values() + '}';
    }

    private class InternalStateChangeListener implements StateChangeListener {

        @Override
        public void stateChanged(CCSTimeStamp when, Enum state, Enum oldState) {
            notifyStateChanged(when, state,oldState);
        }


    }

    /**
     * An implementation of a future which waits for a particular state.
     */
    private class FutureStatus implements Future<Void> {

        private boolean done = false;
        private boolean cancelled = false;
        private final Enum state;

        FutureStatus(Enum state) {
            this.state = state;
        }

        @Override
        public synchronized boolean cancel(boolean mayInterruptIfRunning) {
            cancelled = true;
            waiters.remove(this);
            notifyAll();
            return true;
        }

        @Override
        public boolean isCancelled() {
            return cancelled;
        }

        @Override
        public boolean isDone() {
            return done;
        }

        @Override
        public synchronized Void get() throws InterruptedException, ExecutionException {
            while (!done) {
                wait();
            }
            return null;
        }

        @Override
        public synchronized Void get(long timeout, TimeUnit unit) throws InterruptedException, ExecutionException, TimeoutException {
            if (!done) {
                wait(unit.toMillis(timeout));
            }
            if (!done) {
                throw new TimeoutException("Timeout waiting for state: " + state);
            }
            return null;
        }

        synchronized void done() {
            done = true;
            notifyAll();
        }
    }

}
